@@ -3,14 +3,14 @@ use warnings;
 use 5.010;
 
 package JIRA::Client::Automated;
-$JIRA::Client::Automated::VERSION = '1.1';
+$JIRA::Client::Automated::VERSION = '1.2';
 =head1 NAME
 
 JIRA::Client::Automated - A JIRA REST Client for automated scripts
 
 =head1 VERSION
 
-version 1.1
+version 1.2
 
 =head1 SYNOPSIS
 
@@ -61,6 +61,9 @@ version 1.1
     $jira->close_issue($key, $resolve, $comment); # resolve is the resolution value
     $jira->delete_issue($key);
 
+    $jira->add_issue_watchers($key, $watcher1, ......);
+    $jira->add_issue_labels($key, $label1, ......);
+
 
 =head1 DESCRIPTION
 
@@ -204,7 +207,7 @@ sub new {
     $self->{_ua} = LWP::UserAgent->new();
 
     # cached JSON object for handling conversions
-    $self->{_json} = JSON->new->utf8();
+    $self->{_json} = JSON->new->utf8()->allow_nonref;
 
     return $self;
 }
@@ -523,7 +526,7 @@ sub _get_transition_id {
     my %status_names = map { $_->{to}{name} => $_ } @$transitions;
 
     my @names = (ref $t_name) ? @$t_name : ($t_name);
-    my @trans = map { $trans_names{$_} // $status_names{$_} } @names;
+    my @trans = map { $trans_names{$_} // $status_names{$_} } @names; # // is incompatible with perl <= 5.8
     my $tran = (grep { defined } @trans)[0]; # use the first defined one
 
     if (not defined $tran) {
@@ -648,7 +651,7 @@ This method is a wrapper for L</transition_issue>.
 sub close_issue {
     my ($self, $key, $resolve, $comment, $update_hash) = @_;
 
-    $comment //= 'Issue closed by script';
+    $comment //= 'Issue closed by script'; # // is incompatible with perl <= 5.8
 
     $update_hash ||= {};
 
@@ -718,7 +721,7 @@ sub create_comment {
 
 =head2 search_issues
 
-    my @search_results = $jira->search_issues($jql, 1, 100);
+    my @search_results = $jira->search_issues($jql, 1, 100, $fields);
 
 You've used JQL before, when you did an "Advanced Search" in the JIRA web
 interface. That's the only way to search via the REST API.
@@ -727,6 +730,8 @@ This is a paged method. Pass in the starting result number and number of
 results per page and it will return issues a page at a time. If you know you
 want all of the results, you can use L</"all_search_results"> instead.
 
+Optional parameter $fields is the arrayref containing the list of fields to be returned. 
+
 This method returns a hashref containing up to five values:
 
 =over 3
@@ -776,13 +781,14 @@ For example, to page through all results C<$max> at a time:
 # Note: if $max is > 1000 (set by jira.search.views.default.max in
 # http://jira.example.com/secure/admin/ViewSystemInfo.jspa) then it'll be truncated to 1000 anyway.
 sub search_issues {
-    my ($self, $jql, $start, $max) = @_;
+    my ($self, $jql, $start, $max, $fields) = @_;
 
+	$fields ||= ['*navigable'];
     my $query = {
         jql        => $jql,
         startAt    => $start,
         maxResults => $max,
-        fields     => ['*navigable'],
+        fields     => $fields,
     };
 
     my $query_json = $self->{_json}->encode($query);
@@ -806,7 +812,6 @@ sub search_issues {
 
     my $results = $self->{_json}->decode($response->decoded_content());
 
-    # TODO: make this return a hash labeling the metadata instead of just a list.
     return {
         total  => $$results{total},
         start  => $$results{startAt},
@@ -830,7 +835,7 @@ sub all_search_results {
     my ($self, $jql, $max) = @_;
 
     my $start = 0;
-    $max //= 100; # is a param for testing
+    $max //= 100; # is a param for testing ; // is incompatible with perl <= 5.8
     my $total = 0;
     my (@all_results, @issues, $results);
 
@@ -847,6 +852,25 @@ sub all_search_results {
     return @all_results;
 }
 
+=head2 get_issue_comments
+
+    $jira->get_issue_comments($key);
+
+Returns arryref of all comments to the given issue.
+
+=cut
+
+sub  get_issue_comments { 
+    my ($self, $key) = @_;
+    my $uri = "$self->{auth_url}issue/$key/comment";
+    my $request = GET $uri;
+    my $response = $self->_perform_request($request);
+    my $content = $self->{_json}->decode($response->decoded_content());
+
+    # dereference to get just the comments arrayref
+    my $comments = $content->{comments};
+    return $comments;
+}
 
 =head2 attach_file_to_issue
 
@@ -896,6 +920,157 @@ sub make_browse_url {
     return $self->{url} . 'browse/' . $key;
 }
 
+=head2 get_link_types 
+
+    my $all_link_types = $jira->get_link_types();
+
+Get the arrayref of all possible link types.
+
+=cut
+
+sub get_link_types { 
+    my ($self) = @_;
+	
+    my $uri = "$self->{auth_url}issueLinkType";
+    my $request = GET $uri;
+    my $response = $self->_perform_request($request);
+
+    # dereference to arrayref, for convenience later
+    my $content = $self->{_json}->decode($response->decoded_content());
+    my $link_types = $content->{issueLinkTypes};
+
+    return $link_types;
+}
+
+=head2 link_issues 
+
+    $jira->link_issues($from, $to, $type);
+
+Establish a link of the type named $type from issue key $from to issue key $to .
+Returns nothing on success; structure containing error messages otherwise.
+
+=cut
+
+
+sub link_issues {    
+    my ($self, $from, $to, $type) = @_;
+
+    my $uri = "$self->{auth_url}issueLink/";
+    my $link = { 
+        inwardIssue   => { key  => $to   },
+        outwardIssue  => { key  => $from },
+        type          => { name => $type }, 
+    };
+
+    my $link_json = $self->{_json}->encode($link);
+
+    my $request = POST $uri,
+      Content_Type        => 'application/json',
+      Content             => $link_json;
+
+    my $response = $self->_perform_request($request);
+
+    if($response->code != 201) { 
+        return $self->{_json}->decode($response->decoded_content());
+    }
+    return;
+}
+
+=head2 add_issue_labels
+
+    $jira->add_issue_labels($issue_key, @labels);
+
+Adds one more more labels to the specified issue.
+
+=cut
+
+
+sub add_issue_labels { 
+    my ($self, $issue_key, @labels) = @_;
+    $self->update_issue($issue_key,  {}, { labels => [ map {{ add => $_ }} @labels ] } );	
+}
+
+=head2 remove_issue_labels
+
+    $jira->remove_issue_labels($issue_key, @labels);
+
+Removes one more more labels from the specified issue.
+
+=cut
+
+sub remove_issue_labels { 
+    my ($self, $issue_key, @labels) = @_;
+    $self->update_issue($issue_key,  {}, { labels => [ map {{ remove => $_ }} @labels ] } );	
+}
+
+=head2 add_issue_watchers 
+
+    $jira->add_issue_watchers($key, @watchers);
+
+Adds watchers to the specified issue. Returns nothing if success; otherwise returns a structure containing error message.
+
+=cut
+
+sub add_issue_watchers { 
+    my ($self, $key, @watchers) = @_;
+    my $uri = "$self->{auth_url}issue/$key/watchers";
+    foreach my $w (@watchers) { 
+        my $request = POST $uri,
+            Content_Type    => 'application/json',
+            Content         => $self->{_json}->encode($w);
+        my $response = $self->_perform_request($request);
+        if($response->code != 204) { 
+              return $self->{_json}->decode($response->decoded_content());
+        }
+    }
+    return;
+}
+
+=head2 get_issue_watchers
+
+    $jira->get_issue_watchers($key);
+
+Returns arryref of all watchers of the given issue.
+
+=cut
+
+sub  get_issue_watchers { 
+    my ($self, $key) = @_;
+    my $uri = "$self->{auth_url}issue/$key/watchers";
+    my $request = GET $uri;
+    my $response = $self->_perform_request($request);
+    my $content = $self->{_json}->decode($response->decoded_content());
+
+    # dereference to get just the watchers arrayref
+    my $watchers = $content->{watchers};
+    return $watchers;
+}
+
+=head2 assign_issue
+
+    $jira->assign_issue($key, $assignee_name);
+
+Assigns the issue to that person. Returns the key of the issue if it succeeds.
+
+=cut
+
+sub assign_issue {
+    my ($self, $key, $assignee_name) = @_;
+
+    my $assignee = {};
+    $assignee->{name} = $assignee_name;
+
+    my $issue_json = $self->{_json}->encode($assignee);
+    my $uri        = "$self->{auth_url}issue/$key/assignee";
+
+    my $request = PUT $uri,
+      Content_Type => 'application/json',
+      Content      => $issue_json;
+
+    my $response = $self->_perform_request($request);
+
+    return $key;
+}
 
 =head1 FAQ
 
@@ -938,6 +1113,19 @@ Thanks very much to:
 
 =back
 
+=over 4
+
+=item Ivan E. Panchenko <panchenko@cpan.org>
+
+=back
+
+=encoding utf8
+
+=over 4
+
+=item José Antonio Perez Testa <japtesta@gmail.com>
+
+=back
 
 =head1 COPYRIGHT AND LICENSE
 
